/******************************************************************************/
/*!
\file   Hatching.fx
\author Alex Moquia
\date   3/30/2010
\brief

	This shader implements the cross-hatching effects in the game.

All content  2010 DigiPen (USA) Corporation, all rights reserved.
*/
/******************************************************************************/
#include "global.fx"

//static const float whiteVal = 0.6;
//static const float l1DotVal = 0.4;
//static const float l2DotVal = 0.2;
//static const float l3DotVal = 0.0;
//static const float l4DotVal = -0.2;
//static const float l5DotVal = -0.05;

static const float whiteVal = 0.98;
static const float l1DotVal = 0.9;
static const float l2DotVal = 0.6;
static const float l3DotVal = 0.3;
static const float l4DotVal = -0.5;
static const float l5DotVal = -0.9;

//maps
uniform extern texture gHatching1; //hatching light
uniform extern texture gHatching2; //hatching light
uniform extern texture gHatching3; //hatching light
uniform extern texture gHatching4; //hatching light
uniform extern texture gHatching5; //hatching light

//obtains the number of lights used in the scene
uniform extern int numLights;

sampler Hatching1S = sampler_state
{
	Texture = <gHatching1>;
	MinFilter = ANISOTROPIC;
	MaxAnisotropy = 8;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

sampler Hatching2S = sampler_state
{
	Texture = <gHatching2>;
	MinFilter = ANISOTROPIC;
	MaxAnisotropy = 8;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

sampler Hatching3S = sampler_state
{
	Texture = <gHatching3>;
	MinFilter = ANISOTROPIC;
	MaxAnisotropy = 8;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

sampler Hatching4S = sampler_state
{
	Texture = <gHatching4>;
	MinFilter = ANISOTROPIC;
	MaxAnisotropy = 8;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};
sampler Hatching5S = sampler_state
{
	Texture = <gHatching5>;
	MinFilter = ANISOTROPIC;
	MaxAnisotropy = 8;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};
/*
struct VS_INPUT
{
	float3 position	: POSITION;
	float3 normal : NORMAL;
	float2 tex0 : TEXCOORD0;
	
};

struct VS_INPUT_SKIN
{
    float4 position	: POSITION;
    float3 normal : NORMAL;
    float2 tex0 : TEXCOORD0;
    float4 BlendWeights : BLENDWEIGHT;
    float4 BlendIndices : BLENDINDICES;
};

struct VS_OUTPUT_TRUE
{
    float4 pos : POSITION;
	float3 worldpos : TEXCOORD0;
	float2 tex0  : TEXCOORD1;
    float3 normal : TEXCOORD2;
};

struct VS_OUTPUT_NORMAL_TRUE
{
	float4 pos : POSITION;
	float3 worldpos : TEXCOORD0;
	float2 tex0 : TEXCOORD1;
	float3 normal : TEXCOORD2;
	float3 tangent : TEXCOORD3;
	float3 binormal : TEXCOORD4;
};
*/

void TransformVerticesHatching(inout float3 Pos, inout float3 Normal)
{	
    float3x3 rotation;
    rotation[0][0] = world[0][0];
    rotation[0][1] = world[0][1];
    rotation[0][2] = world[0][2];
    
    rotation[1][0] = world[1][0];
    rotation[1][1] = world[1][1];
    rotation[1][2] = world[1][2];
    
    rotation[2][0] = world[2][0];
    rotation[2][1] = world[2][1];
    rotation[2][2] = world[2][2];
    
	Pos = mul(Pos, rotation); // changes here
	
	float3x3 rotation2;
    rotation2[0][0] = worldViewProj[0][0];
    rotation2[0][1] = worldViewProj[0][1];
    rotation2[0][2] = worldViewProj[0][2];
    
    rotation2[1][0] = worldViewProj[1][0];
    rotation2[1][1] = worldViewProj[1][1];
    rotation2[1][2] = worldViewProj[1][2];
    
    rotation2[2][0] = worldViewProj[2][0];
    rotation2[2][1] = worldViewProj[2][1];
    rotation2[2][2] = worldViewProj[2][2];
    
	Normal = mul(Normal, rotation2);	
    Normal = normalize(Normal);	
	Pos.x += world[3][0];
	Pos.y += world[3][1];
	Pos.z += world[3][2];
}

void TransformSkinnedVerticesHatching(inout float3 Pos, inout float3 Normal, in float4 origPos, in float3 origNormal,
							in float4 BlendWeights, in float4 BlendIndices, in int NumBones)
{
    float3x3 rotation;
    rotation[0][0] = world[0][0];
    rotation[0][1] = world[0][1];
    rotation[0][2] = world[0][2];
    
    rotation[1][0] = world[1][0];
    rotation[1][1] = world[1][1];
    rotation[1][2] = world[1][2];
    
    rotation[2][0] = world[2][0];
    rotation[2][1] = world[2][1];
    rotation[2][2] = world[2][2];
    	
   
    float BlendWeightsArray[4] = (float[4])BlendWeights;
    int   IndexArray[4]        = (int[4])BlendIndices;
	    
	for (int iBone = 0; iBone < NumBones ; iBone++)
    {       
        Pos += mul( origPos , WorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
        Normal += mul(origNormal , WorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
    }
        
	Normal = normalize(Normal);
	Pos = mul(Pos, rotation);
	Pos.x += world[3][0];
	Pos.y += world[3][1];
	Pos.z += world[3][2];
}



/* A light with increasing radius */
void TechHatching(in int i, in float3 worldPos, inout float4 ambient)
{
	float r = dirLights[i].radius * saturate((gTime - dirLights[i].activationTime )/20.0f);
	float3 lightDir = normalize(dirLights[i].pos - worldPos);
	float distanceToLight = length(dirLights[i].pos - worldPos);
	float4 atten = saturate(1.0f - (distanceToLight/r));
	
	//ambient light computation
	float4 color = float4(0.0f, 0.0f, 0.0f,1.0f);
	float colorScale = (1-saturate(atten*15));
	ambient += color * colorScale;
	ambient.a = 1.0f - ambient.a;	
}



VS_OUTPUT_TRUE VS_Hatching(VS_INPUT IN)
{

	//VS_OUTPUT_TRUE OUT;
	VS_OUTPUT_TRUE OUT = (VS_OUTPUT_TRUE)0;

	float3 Pos = IN.position;
    float3 Normal = IN.normal;
    //Normal = float3(0.0f, 1.0f, 0.0f);
    	
	TransformVerticesHatching(Pos, Normal);

	OUT.pos = mul( float4(Pos, 1.0f) , worldViewProj);
	OUT.normal = Normal;
	//OUT.normal = mul( float4(Normal, 0) , world );

	//transform vertex position to world space
	OUT.worldpos = Pos;

	//OUT.tex0 = (Pos.xy/10)+IN.tex0;
	OUT.tex0 = IN.tex0;
        
	return OUT;
	
}

//float GetBlendFactor
//x = val - lower

float4 PS_Hatching(VS_OUTPUT_TRUE IN) : COLOR
{
	float4 ambient = float4(0, 0, 0, 0);
	float3 diffuse = float3(0, 0, 0);
	float3 specular = float3(0, 0, 0);

	float3 viewDir = gEyePosW - IN.worldpos;

	float3 n = normalize(IN.normal)*-1.0f;
	
	float3 lightDir = float3(0,-1,0);
	
	float val = dot(lightDir, n);	
	
	float4 hatchColor;
	float4 white = float4(1,1,1,0);
	
	//Debug colors
	float4 black = float4(0,0,0,0);
	float4 red = float4(1,0,0,0);
	float4 green = float4(0,1,0,0);
	float4 blue = float4(0,0,1,0);
	float4 yellow = float4(1,1,0,0);
	float4 cyan = float4(0,1,1,0);
	float4 pink = float4(1,0,1,0);
	float4 brown = float4(0.7,0.5,0,0);	
	float4 gray1 = float4(0.5,0.5,0.5,0);
	float4 gray2 = float4(0.4,0.4,0.4,0);
	float4 gray3 = float4(0.3,0.3,0.3,0);
	float4 gray4 = float4(0.2, 0.2, 0.2, 0);
	float4 gray5 = float4(0.1,0.1,0.1,0);
	
	
	
	float4 l1 = tex2D(Hatching1S, IN.tex0);
	float4 l2 = tex2D(Hatching2S, IN.tex0);
	float4 l3 = tex2D(Hatching3S, IN.tex0);
	float4 l4 = tex2D(Hatching4S, IN.tex0);
	float4 l5 = tex2D(Hatching5S, IN.tex0);

	/*if(val > whiteVal)
		hatchColor = white;
	else
	if(val > l1DotVal) 
		hatchColor = lerp(l1,white, white*((val - l1DotVal) - (whiteVal-val))); 
	else
	*/
 	if(val > l2DotVal)
		hatchColor = lerp(l2,l1, l1*((val - l2DotVal) - (l1DotVal-val))); //		output = tex2D(HatchingMediumS, IN.tex0);
	else
	if(val > l3DotVal)
		hatchColor = lerp(l3,l2,  l2*((val-l3DotVal) - (l2DotVal-val)));  //output = tex2D(HatchingDarkS, IN.tex0);
	else
	//if(val > l4DotVal)
		hatchColor = lerp(l4,l3,  l3*((val-l4DotVal) - (l3DotVal-val)));
	//else
	//	hatchColor = lerp(l5,l4,  l4*((val-l5DotVal) - (l4DotVal-val)));
		
		
	/* else
	
	if(val > 0.75)
		output = red;
	else
	if(val >0.5)
		output = green;
	else
	if(val >0.25)
		output = blue;
	else
	if(val >0.15)
		output = yellow;
	else
	if(val > 0.05)
		output = cyan;
	else
	if(val > 0.01)
		output = pink;
	else
	if(val > 0)
		output = brown;
	else
	if(val == 0.0)
		output = white;
	else
	if(val > -0.0000000001)
		output = gray2; 
	else
	if(val > -0.000000001)
		output = gray3;
	else
	if(val >-0.000000009)
		output = gray4;
	else
	if( val > -0.75)
		output = gray5; 
	else
		output = black;
		
*/
		
	//output = float4(n * 0.5 + 0.5,1);
	
	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional = 3 , point = 1, or spotlight = 2
		TechHatching(i, IN.worldpos,ambient);
    }	
	
	//zero out the hatch color alpha
	hatchColor.a = 0.0f;
	
	hatchColor = saturate(ambient + hatchColor);
	
	return hatchColor;
}

technique Hatching
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_Hatching();
		PixelShader = compile ps_3_0 PS_Hatching();
		AlphaBlendEnable = True;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;		
	}
}
