/******************************************************************************/
/*!
\file   Instance.fx
\author Jonathan Hurtado
\date   2/15/2010
\brief

	This shader manipulates the vertices collected via Hardware Instacing

All content  2010 DigiPen (USA) Corporation, all rights reserved.
*/
/******************************************************************************/

//#include "global.fx"
#include "Hatching.fx"
//#include "NormalMap.fx"

//maps
uniform extern texture gTex; //diffuse
uniform extern texture gNormalMap; //normal
uniform extern texture gSpecularMap; //specular
uniform extern texture gAlphaMap;  //alpha

//booleans used to determine if model had diffuse and specular textures
uniform extern bool hasDiffuseMap;
uniform extern bool	hasSpecularMap;
uniform extern bool isHatching;

//the global ambient color of the scene
uniform extern float4 globalAmbient;

//obtains the number of lights used in the scene
//uniform extern int numLights;

sampler TexS = sampler_state
{
	Texture = <gTex>;
	MinFilter = ANISOTROPIC;
	MaxAnisotropy = 8;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
	AddressV  = WRAP;
};

sampler NormalMapS = sampler_state
{
	Texture = <gNormalMap>;
	MinFilter = ANISOTROPIC;
	MaxAnisotropy = 8;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
	AddressV  = WRAP;
};

sampler SpecularMapS = sampler_state
{
    Texture = <gSpecularMap>;
    MinFilter = ANISOTROPIC;
    MaxAnisotropy = 8;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;  
};

sampler AlphaMapS = sampler_state
{
    Texture = <gAlphaMap>;
    MinFilter = ANISOTROPIC;
    MaxAnisotropy = 8;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU = WRAP;
	AddressV = WRAP;  
};

struct VS_INPUT_PRIMITIVE_INSTANCE
{
	float3 position	: POSITION0;
	float3 normal : NORMAL;
	float2 tex0 : TEXCOORD0;
	float3 color : COLOR0;

	float4 column1 : POSITION1;
	float4 column2 : POSITION2;
	float4 column3 : POSITION3;
};

struct VS_INPUT_DEFAULT_INSTANCE
{
	float3 position	: POSITION0;
	float3 normal : NORMAL;
	float2 tex0 : TEXCOORD0;

	float4 column1 : POSITION1;
	float4 column2 : POSITION2;
	float4 column3 : POSITION3;
};

struct VS_INPUT_NORMAL_INSTANCE
{
	float3 position	: POSITION0;
	float3 normal : NORMAL;
	float3 tangent  : TANGENT0;
    float3 binormal : BINORMAL0;
	float2 tex0 : TEXCOORD0;

	float4 column1 : POSITION1;
	float4 column2 : POSITION2;
	float4 column3 : POSITION3;
};
/*
struct VS_OUTPUT_TRUE
{
    float4 pos : POSITION;
	float3 worldpos : TEXCOORD0;
	float2 tex0  : TEXCOORD1;
    float3 normal : TEXCOORD2;
};
*/

struct VS_OUTPUT_NORMAL_TRUE
{
	float4 pos : POSITION;
	float3 worldpos : TEXCOORD0;
	float2 tex0 : TEXCOORD1;
	float3 normal : TEXCOORD2;
	float3 tangent : TEXCOORD3;
	float3 binormal : TEXCOORD4;
};

//struct VS_OUTPUT
//{
//	float4 position : POSITION;
//	float3 worldPos : TEXCOORD0;
//	float2 texCoord : TEXCOORD1;
//	float3 viewDir : TEXCOORD2;
//	float3 normal : TEXCOORD3;
//};

void TransformVertices(inout float3 Pos, inout float3 Normal, in float4x4 world2)
{	
    float3x3 rotation;
    rotation[0][0] = world2[0][0];
    rotation[0][1] = world2[0][1];
    rotation[0][2] = world2[0][2];
    
    rotation[1][0] = world2[1][0];
    rotation[1][1] = world2[1][1];
    rotation[1][2] = world2[1][2];
    
    rotation[2][0] = world2[2][0];
    rotation[2][1] = world2[2][1];
    rotation[2][2] = world2[2][2];
    
    Normal = normalize(Normal);
	Pos = mul(Pos, rotation); // changes here
	Pos.x += world2[3][0];
	Pos.y += world2[3][1];
	Pos.z += world2[3][2];
}


void TransformVerticesInstanceHatching(inout float3 Pos, inout float3 Normal, in float4x4 world2)
{	
    float3x3 rotation;
    rotation[0][0] = world2[0][0];
    rotation[0][1] = world2[0][1];
    rotation[0][2] = world2[0][2];
    
    rotation[1][0] = world2[1][0];
    rotation[1][1] = world2[1][1];
    rotation[1][2] = world2[1][2];
    
    rotation[2][0] = world2[2][0];
    rotation[2][1] = world2[2][1];
    rotation[2][2] = world2[2][2];
    
    float3x3 rotation2;
    rotation2[0][0] = worldViewProj[0][0];
    rotation2[0][1] = worldViewProj[0][1];
    rotation2[0][2] = worldViewProj[0][2];
    
    rotation2[1][0] = worldViewProj[1][0];
    rotation2[1][1] = worldViewProj[1][1];
    rotation2[1][2] = worldViewProj[1][2];
    
    rotation2[2][0] = worldViewProj[2][0];
    rotation2[2][1] = worldViewProj[2][1];
    rotation2[2][2] = worldViewProj[2][2];
    
    Normal = mul(Normal, rotation2);
    Normal = normalize(Normal);
	Pos = mul(Pos, rotation); // changes here
	Pos.x += world2[3][0];
	Pos.y += world2[3][1];
	Pos.z += world2[3][2];
}    
    
void DirectionalLight(in int i, in float3 n, in float3 viewDir, in float2 texCoord,
						inout float3 ambient, inout float3 diffuse, inout float3 specular)
{
	//float3 lightDir = gLightPosW - worldPos;
	//float3 lightDir = -gLightDirW;
	float3 lightDir = -(dirLights[i].direction);

	float3 halfVector = normalize(normalize(lightDir) + normalize(viewDir));

    float3 h = normalize(halfVector);
    float3 l = normalize(lightDir);

	//ambient light computation
	ambient += (dirLights[i].ambient).rgb;
	
	//diffuse light computation
	//n.x = sin(halfVector.x*180);
	//n.y = cos(halfVector.y*180);
	//n = normalize(n);
	float nDotL = dot(n, l);
	diffuse += (dirLights[i].diffuse).rgb * nDotL;
	//diffuse += float3(0.0f, 1.0f, 0.0f);
	//diffuse = n;
	//specular light computation
	float nDotH = saturate(dot(n, h));
	float power = 0.0f;
	//only calculate specular if nDotL is not equal to 0
	if (nDotL != 0.0f)
	{
		if (hasSpecularMap)
		{
			//power = pow(nDotH, tex2D(gSpecularMap, texCoord).r);
			power = pow(nDotH, 100); 
		}
		else
		{
			power = pow(nDotH, 100); 
		}
	}
	specular += (dirLights[i].specular).rgb * power;

	//float power = (nDotL == 0.0f) ? 0.0f : pow(nDotH, gSpecPower);

	//might use this code to handle specular maps
	//     if (nDotVP == 0.0)
	//         pf = 0.0;
	//     else
	//         pf = pow(nDotHV, gl_FrontMaterial.shininess);
	//         pf = pow(nDotHV, texture2D(specMap, texCoord).r);

}

void DirectionalLightNormal(in int i, in float3 n, in float3 viewDir, in float3x3 tbnMatrix, in float2 texCoord,
								inout float3 ambient, inout float3 diffuse, inout float3 specular)
{
	//float3 lightDir = gLightPosW - worldPos;
	//float3 lightDir = -gLightDirW;
	float3 lightDir = -(dirLights[i].direction);
	
	float3 halfVector = (normalize(normalize(lightDir) + normalize(viewDir)))/2.0f;

	//float3 viewDirT = mul(halfVector, tbnMatrix);
	//float3 lightDirT = mul(lightDir, tbnMatrix);

    float3 h = normalize(halfVector);	// replaced viewDirT
    float3 l = normalize(lightDir);		// replaced lightDirT
	//n = mul(n, tbnMatrix);
	//ambient light computation
	ambient += (dirLights[i].ambient).rgb;
	
	//diffuse light computation
	float nDotL = saturate(dot(n, l));
	diffuse += (dirLights[i].diffuse).rgb * nDotL;
	//diffuse += float3(1.0f, 0.0f, 1.0f);
	//diffuse = n;
	//specular light computation
	float nDotH = saturate(dot(n, h));
	float power = 0.0f;
	//only calculate specular if nDotL is not equal to 0
	if (nDotL != 0.0f)
	{
		if (hasSpecularMap)
		{
			//power = pow(nDotH, tex2D(gSpecularMap, texCoord).r);
			power = pow(nDotH, 100); 
		}
		else
		{
			power = pow(nDotH, 100); 
		}
		
	}
	specular += (dirLights[i].specular).rgb * power;

//might use this code to handle specular maps
//     if (nDotVP == 0.0)
//         pf = 0.0;
//     else
//         pf = pow(nDotHV, gl_FrontMaterial.shininess);
//         pf = pow(nDotHV, texture2D(specMap, texCoord).r);

}

void PointLight(in int i, in float3 n, in float3 worldPos, in float3 viewDir, in float2 texCoord,
						inout float3 ambient, inout float3 diffuse, inout float3 specular)
{
	//float3 lightDirT = mul((dirLights[i].pos - worldPos) / dirLights[i].radius, tbnMatrix);
	//float3 atten = 1.0f - dot(lightDirT, lightDirT);

	float3 lightDir = normalize(dirLights[i].pos - worldPos);
	float distanceToLight = length(dirLights[i].pos - worldPos);
	float3 atten = saturate(1.0f - (distanceToLight/dirLights[i].radius));
	
	//n = -1*n;
	float3 viewDirT = (viewDir + lightDir) / 2.0f; // added new line
	//float3 viewDirT = mul(viewDir, tbnMatrix);
	//n = mul(n, tbnMatrix);
    float3 l =  normalize(worldPos - dirLights[i].pos);
    float3 h = normalize(viewDirT);

	//ambient light computation
	ambient += (dirLights[i].ambient).rgb * atten + globalAmbient;
	
	//make sure nDotL is clamped between 0 and 1
	float nDotL =  saturate(dot(-n,l));
	diffuse += (dirLights[i].diffuse).rgb * nDotL * atten;
	//diffuse += float3(1.0f, 0.0f, 0.0f);
	
	//specular light computation
	//make sure nDotH is clamped between 0 and 1
	float nDotH = saturate(dot(n, h));
	float power = 0.0f;
	
	//only calculate specular if nDotL is not equal to 0
	if (nDotL != 0.0f)
	{
		if (hasSpecularMap)
		{
			//power = pow(nDotH, gSpecPower); 
			power = pow(nDotH, 100); 
		}
		else
		{
			power = pow(nDotH, 100); 
		}
	}
	specular += (dirLights[i].specular).rgb * power * atten;
	//specular = 0;

	//might use this code to handle specular maps
	//     if (nDotVP == 0.0)
	//         pf = 0.0;
	//     else
	//         pf = pow(nDotHV, gl_FrontMaterial.shininess);
	//         pf = pow(nDotHV, texture2D(specMap, texCoord).r);

}

void PointLightNormal(in int i, in float3 n, in float3 worldPos, in float3 viewDir, in float3x3 tbnMatrix,
								in float2 texCoord, inout float3 ambient, inout float3 diffuse, inout float3 specular)
{// THIS IS THE FUNCTION I'M USING 

	//float3 lightDirT = mul((dirLights[i].pos - worldPos) / dirLights[i].radius, tbnMatrix);
	//float3 atten = 1.0f - dot(lightDirT, lightDirT);

	float3 lightDir = normalize(dirLights[i].pos - worldPos);
	float distanceToLight = length(dirLights[i].pos - worldPos);
	float3 atten = saturate(1.0f - (distanceToLight/dirLights[i].radius));
	
	//n = -1*n;
	float3 viewDirT = (viewDir + lightDir) / 2.0f; // added new line
	//float3 viewDirT = mul(viewDir, tbnMatrix);
	//n = mul(n, tbnMatrix);
    float3 l =  normalize(worldPos - dirLights[i].pos);
    float3 h = normalize(viewDirT);

	//ambient light computation
	ambient += (dirLights[i].ambient).rgb * atten + globalAmbient;
	
	//make sure nDotL is clamped between 0 and 1
	float nDotL =  saturate(dot(-n,l));
	diffuse += (dirLights[i].diffuse).rgb * nDotL * atten;
	//diffuse += float3(1.0f, 0.0f, 0.0f);
	
	//specular light computation
	//make sure nDotH is clamped between 0 and 1
	float nDotH = saturate(dot(n, h));
	float power = 0.0f;
	
	//only calculate specular if nDotL is not equal to 0
	if (nDotL != 0.0f)
	{
		if (hasSpecularMap)
		{
			//power = pow(nDotH, gSpecPower); 
			//power = pow(nDotH, tex2D(gSpecularMap, texCoord).r);
			power = pow(nDotH, 100); 
		}
		else
		{
			power = pow(nDotH, 100); 
		}
	}
	specular += (dirLights[i].specular).rgb * power * atten;
}

float4 PS_Blinn_Phong(VS_OUTPUT_TRUE IN) : COLOR
{
	float4 color;

	float3 ambient = float3(0, 0, 0);
	float3 diffuse = float3(0, 0, 0);
	float3 specular = float3(0, 0, 0);

	float3 n = normalize(IN.normal);
	
	float3 viewDir = gEyePosW - IN.worldpos;

	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional, point, or spotlight
	    if (dirLights[i].type == 3)
		{
    	    DirectionalLight(i, n, viewDir, IN.tex0, ambient, diffuse, specular);
		}
	    else if (dirLights[i].type == 1)
		{
			PointLight(i, n, IN.worldpos, viewDir, IN.tex0, ambient, diffuse, specular);
		}
	    //else
	        //SpotLight(i, eye, ecPosition3, normal, amb, diff, spec);
	}

	//ambient light computation
	ambient = saturate(ambient);
	diffuse = saturate(diffuse);
	specular = saturate(specular);
	ambient *= (gAmbientMtrl).rgb;
	diffuse *= (gDiffuseMtrl).rgb;
	specular *= (gSpecularMtrl).rgb;
		
	color.rgb = ambient + diffuse + specular;
	color.a = gDiffuseMtrl.a;

	if (hasDiffuseMap)
	{
		return color * tex2D(TexS, IN.tex0);
	}
	else
	{
		//return float4(IN.normal, 1.0f);
		return color;
	}
	
}

float4 PS_Blinn_Phong_Alpha(VS_OUTPUT_TRUE IN) : COLOR
{
	float4 color;

	float3 ambient = float3(0, 0, 0);
	float3 diffuse = float3(0, 0, 0);
	float3 specular = float3(0, 0, 0);

	float3 n = normalize(IN.normal);

	float3 viewDir = gEyePosW - IN.worldpos;

	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional, point, or spotlight
	    if (dirLights[i].type == 3)
		{
    	    DirectionalLight(i, n, viewDir, IN.tex0, ambient, diffuse, specular);
		}
	    else if (dirLights[i].type == 1)
		{
			PointLight(i, n, IN.worldpos, viewDir, IN.tex0, ambient, diffuse, specular);
		}
	    //else
	        //SpotLight(i, eye, ecPosition3, normal, amb, diff, spec);
	}

	//ambient light computation
	ambient = saturate(ambient);
	diffuse = saturate(diffuse);
	specular = saturate(specular);
	ambient *= (gAmbientMtrl).rgb;
	diffuse *= (gDiffuseMtrl).rgb;
	specular *= (gSpecularMtrl).rgb;
		
	color.rgb = ambient + diffuse + specular;

	if (hasDiffuseMap)
	{
		color *= tex2D(TexS, IN.tex0);
	}
	color.a = tex2D(AlphaMapS, IN.tex0).r;
	return color;
}

float4 PS_NormalMap(VS_OUTPUT_NORMAL_TRUE IN) : COLOR
{	// USING THIS PIXEL SHADER 
	float4 color;
	
	float3 ambient = float3(0, 0, 0);
	float3 diffuse = float3(0, 0, 0);
	float3 specular = float3(0, 0, 0);

	float3 viewDir = gEyePosW - IN.worldpos;

	float3 n = normalize(tex2D(NormalMapS, IN.tex0).rgb * 2.0f - 1.0f);
	//float3 n = normalize(IN.normal);
	//n = mul(float4(n,1.0f),world).rgb;
	// Build TBN-basis.
	float3x3 TBN;
	TBN[0] = IN.tangent;
	TBN[1] = IN.binormal;
	TBN[2] = IN.normal;
	
	float3x3 tbnMatrix = transpose(TBN);

	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional, point, or spotlight
	    if (dirLights[i].type == 3)
		{
    	    DirectionalLightNormal(i, n, viewDir, tbnMatrix, IN.tex0, ambient, diffuse, specular);
	    }
		else if (dirLights[i].type == 1)
		{
			PointLightNormal(i, n, IN.worldpos, viewDir, tbnMatrix, IN.tex0, ambient, diffuse, specular);
		}
	    //else
	        //SpotLight(i, eye, ecPosition3, normal, amb, diff, spec);		
    }

	//ambient light computation
	//ambient = saturate(ambient);
	diffuse = saturate(diffuse);
	//specular = saturate(specular);
	ambient *= (gAmbientMtrl).rgb;
	diffuse *= (gDiffuseMtrl).rgb;
	specular *= (gSpecularMtrl).rgb;
		
	color.rgb = ambient + diffuse + specular;
	color.a = gDiffuseMtrl.a;

	if (hasDiffuseMap)
	{
		return color * tex2D(TexS, IN.tex0);
	}
	else
	{
		return color;
	}
}

float4 PS_Toon(VS_OUTPUT_NORMAL_TRUE IN) : COLOR
{

	float4 color = (float4)0;
	if (hasDiffuseMap)
	{
		color = tex2D(TexS, IN.tex0);
	}

	float3 n = normalize(IN.normal);

	float3 viewDir = gEyePosW - IN.worldpos;


	 // Calc light direction and intensity per pixel

	float3 lightDir;

	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional, point, or spotlight
	    if (dirLights[i].type == 3)
		{
    	    lightDir = normalize(-(dirLights[i].direction));
		}
	    //else
	        //SpotLight(i, eye, ecPosition3, normal, amb, diff, spec);
	}

	
	float intensity = max(0, dot(lightDir,n));

	// Reduce color palette by flooring values at discrete intervals.

	color.rgba -= (color.rgba %  0.1f); //discretePallete

	// Darken color based on intensity

	if (intensity > 0.95)
		color -= float4(0.0,0.0,0.0,0.0);
	else if (intensity > 0.5)
		color -= float4(0.2,0.2,0.2,0.0);
	else if (intensity > 0.25)
		color -= float4(0.4,0.4,0.4,0.0);
	else
		color -= float4(0.6,0.6,0.6,0.0);

	return color;
}


float4 PS_NormalMap_Alpha(VS_OUTPUT_NORMAL_TRUE IN) : COLOR
{
	float4 color;
	
	float3 ambient = float3(0, 0, 0);
	float3 diffuse = float3(0, 0, 0);
	float3 specular = float3(0, 0, 0);

	float3 n = normalize(tex2D(NormalMapS, IN.tex0).rgb * 2.0f - 1.0f);

	float3 viewDir = gEyePosW - IN.worldpos;

	// Build TBN-basis.
	float3x3 TBN;
	TBN[0] = IN.tangent;
	TBN[1] = IN.binormal;
	TBN[2] = IN.normal;
	
	float3x3 tbnMatrix = transpose(TBN);

	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional, point, or spotlight
	    if (dirLights[i].type == 3)
		{
    	    DirectionalLightNormal(i, n, viewDir, tbnMatrix, IN.tex0, ambient, diffuse, specular);
	    }
		else if (dirLights[i].type == 1)
		{
			PointLightNormal(i, n, IN.worldpos, viewDir, tbnMatrix, IN.tex0, ambient, diffuse, specular);
		}
	    //else
	        //SpotLight(i, eye, ecPosition3, normal, amb, diff, spec);		
    }

	//ambient light computation
	//ambient = saturate(ambient);
	diffuse = saturate(diffuse);
	//specular = saturate(specular);
	ambient *= (gAmbientMtrl).rgb;
	diffuse *= (gDiffuseMtrl).rgb;
	specular *= (gSpecularMtrl).rgb;
		
	color.rgb = ambient + diffuse + specular;

	if (hasDiffuseMap)
	{
		color *= tex2D(TexS, IN.tex0);
	}
	color.a = tex2D(AlphaMapS, IN.tex0).r;
	return color;

}

//-----------------------------------------------------------------------------
// Name: VS_HWInstancing
// Type: Vertex shader (HW Instancing)
// Desc: This shader computes standard transform and lighting for unlit, texture-mapped triangles.
//-----------------------------------------------------------------------------
VS_OUTPUT_TRUE VS_HWInstancing( VS_INPUT_DEFAULT_INSTANCE IN )
{

	VS_OUTPUT_TRUE OUT;
	float4x4 world2 =
	{
		IN.column1.x, IN.column2.x, IN.column3.x, 0.0f,
		IN.column1.y, IN.column2.y, IN.column3.y, 0.0f,
		IN.column1.z, IN.column2.z, IN.column3.z, 0.0f,
		IN.column1.w, IN.column2.w, IN.column3.w, 1.0f,
	};

	float3 Pos = IN.position;
    float3 Normal = IN.normal;
    //Normal = float3(0.0f, 1.0f, 0.0f);
    	
    //if(isHatching)
		TransformVerticesInstanceHatching(Pos,Normal,world2);
	//else
	//	TransformVertices(Pos, Normal, world2);
	
	OUT.pos = mul( float4(Pos, 1.0f) , worldViewProj);
	OUT.normal = mul( float4(Normal, 0) , world2 );
	OUT.worldpos = Pos;
	OUT.tex0 = IN.tex0;

	return OUT;
	
}

VS_OUTPUT_NORMAL_TRUE VS_HWInstancingNormal( VS_INPUT_NORMAL_INSTANCE IN )
{
	VS_OUTPUT_NORMAL_TRUE OUT;

	float4x4 world2 =
	{
		IN.column1.x, IN.column2.x, IN.column3.x, 0.0f,
		IN.column1.y, IN.column2.y, IN.column3.y, 0.0f,
		IN.column1.z, IN.column2.z, IN.column3.z, 0.0f,
		IN.column1.w, IN.column2.w, IN.column3.w, 1.0f,
	};

	float3 Pos = IN.position;
    float3 Normal = IN.normal;
    	
	TransformVertices(Pos, Normal, world2);
	
	
	//OUT.worldpos = mul(float4(IN.position, 1.0f), world).xyz;
	//OUT.worldpos = mul(float4(Pos, 1.0f), world).xyz;
	OUT.worldpos = Pos;

	//OUT.viewDir = gEyePosW - OUT.worldpos;
		
	OUT.pos = mul(float4(Pos, 1.0f), worldViewProj);
	OUT.tex0 = IN.tex0;

	//OUT.normal = mul( float4(Normal, 0) , world2 );
	OUT.normal = IN.normal; // Change Here
	OUT.tangent = IN.tangent;
	OUT.binormal = IN.binormal;

	return OUT;
}

technique InstanceDraw
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_HWInstancing();
		PixelShader = compile ps_3_0 PS_Blinn_Phong();
	}
}

technique InstanceDrawHatching
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_HWInstancing();
		PixelShader = compile ps_3_0 PS_Hatching();
		AlphaBlendEnable = True;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;	
	}
}

technique InstanceDrawAlpha
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_HWInstancing();
		PixelShader = compile ps_3_0 PS_Blinn_Phong_Alpha();
	}
}

technique InstanceDrawNormal
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_HWInstancingNormal();
		//PixelShader = compile ps_3_0 PS_NormalMap();
		PixelShader = compile ps_3_0 PS_Toon();
	}
}

technique InstanceDrawNormalAlpha
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_HWInstancingNormal();
		PixelShader = compile ps_3_0 PS_NormalMap_Alpha();	
	}
}
