/******************************************************************************/
/*!
\file   NormalMap.fx
\author Jonathan Hurtado
\date   1/30/2010
\brief

	This shader implements per-pixel Blinn-Phong and per-pixel normal map shading.

All content  2010 DigiPen (USA) Corporation, all rights reserved.
*/
/******************************************************************************/
/*
float4x4 worldViewProj : WorldViewProjection;
float4x4 world : World;
static const int MaxMatrices = 80;
float4x3   WorldMatrixArray[MaxMatrices] : WORLDMATRIXARRAY;

struct light
{
	float3 pos;     // position of the light in world space
	float3 direction;  // direction of light (used for direction and point lights)
	float4 ambient;  // ambient color of light
	float4 diffuse;  // diffuse color of light
	float4 specular; // specular color of light
	float radius;    // radius of light
	float spotPower; // power of the spotlight
	int type;  // 1 = point light, 2 = spotlight, 3 = directional light
};
*/

#include "global.fx"


//maps
uniform extern texture gTex; //diffuse
uniform extern texture gNormalMap; //normal
uniform extern texture gSpecularMap; //specular
uniform extern texture gAlphaMap;  //alpha
uniform extern texture gHatching; //hatching

//booleans used to determine if model had diffuse and specular textures
uniform extern bool hasDiffuseMap;
uniform extern bool	hasSpecularMap;

//the global ambient color of the scene
uniform extern float4 globalAmbient;

//obtains the number of lights used in the scene
uniform extern int numLights;

sampler TexS = sampler_state
{
	Texture = <gTex>;
	//mINfILTER = anisotropic;
	//mAXaNISOTROPY = 8;
	//mAGfILTER = linear;
	//mIPfILTER = linear;
	//aDDRESSu  = wrap;
    //aDDRESSv  = wrap;

	AddressU = MIRROR;
	AddressV = MIRROR;
	AddressW = MIRROR;
	MIPFILTER = ANISOTROPIC;
	MINFILTER = ANISOTROPIC;
	MAGFILTER = ANISOTROPIC;
};

sampler NormalMapS = sampler_state
{
	Texture = <gNormalMap>;
	MinFilter = ANISOTROPIC;
	MaxAnisotropy = 8;
	MagFilter = LINEAR;
	MipFilter = LINEAR;
	AddressU  = WRAP;
    AddressV  = WRAP;
};

sampler SpecularMapS = sampler_state
{
    Texture = <gSpecularMap>;
    MinFilter = ANISOTROPIC;
    MaxAnisotropy = 8;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;  
};

sampler AlphaMapS = sampler_state
{
    Texture = <gAlphaMap>;
    MinFilter = ANISOTROPIC;
    MaxAnisotropy = 8;
    MagFilter = LINEAR;
    MipFilter = LINEAR;
    AddressU = WRAP;
    AddressV = WRAP;  
};

struct VS_INPUT_NORMAL
{
	float3 position	: POSITION;
	float3 normal : NORMAL;
	float3 tangent  : TANGENT0;
    float3 binormal : BINORMAL0;
	float2 tex0 : TEXCOORD0;
	
};

struct VS_INPUT_SKIN_NORMAL
{
    float4 position	: POSITION;
    float3 normal : NORMAL;
    float3 tangent  : TANGENT0;
    float3 binormal : BINORMAL0;
    float2 tex0 : TEXCOORD0;
    float4 BlendWeights : BLENDWEIGHT;
    float4 BlendIndices : BLENDINDICES;
};

struct VS_OUTPUT_NORMAL_TRUE
{
	float4 pos : POSITION;
	float3 worldpos : TEXCOORD0;
	float2 tex0 : TEXCOORD1;
	float3 normal : TEXCOORD2;
	float3 tangent : TEXCOORD3;
	float3 binormal : TEXCOORD4;
};


void TransformVertices(inout float3 Pos, inout float3 Normal)
{	
    float3x3 rotation;
    rotation[0][0] = world[0][0];
    rotation[0][1] = world[0][1];
    rotation[0][2] = world[0][2];
    
    rotation[1][0] = world[1][0];
    rotation[1][1] = world[1][1];
    rotation[1][2] = world[1][2];
    
    rotation[2][0] = world[2][0];
    rotation[2][1] = world[2][1];
    rotation[2][2] = world[2][2];
    
	Pos = mul(Pos, rotation); // changes here

	Normal = mul(Normal, rotation);	
    Normal = normalize(Normal);	
	Pos.x += world[3][0];
	Pos.y += world[3][1];
	Pos.z += world[3][2];
}

void TransformSkinnedVertices(inout float3 Pos, inout float3 Normal, in float4 origPos, in float3 origNormal,
							in float4 BlendWeights, in float4 BlendIndices, in int NumBones)
{
    float3x3 rotation;
    rotation[0][0] = world[0][0];
    rotation[0][1] = world[0][1];
    rotation[0][2] = world[0][2];
    
    rotation[1][0] = world[1][0];
    rotation[1][1] = world[1][1];
    rotation[1][2] = world[1][2];
    
    rotation[2][0] = world[2][0];
    rotation[2][1] = world[2][1];
    rotation[2][2] = world[2][2];
    	
   
    float BlendWeightsArray[4] = (float[4])BlendWeights;
    int   IndexArray[4]        = (int[4])BlendIndices;
	    
	for (int iBone = 0; iBone < NumBones ; iBone++)
    {       
        Pos += mul( origPos , WorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
        Normal += mul(origNormal , WorldMatrixArray[IndexArray[iBone]]) * BlendWeightsArray[iBone];
    }
        
	Normal = normalize(Normal);
	Pos = mul(Pos, rotation);
	Pos.x += world[3][0];
	Pos.y += world[3][1];
	Pos.z += world[3][2];
}

void DirectionalLight(in int i, in float3 n, in float3 viewDir, in float2 texCoord,
						inout float3 ambient, inout float3 diffuse, inout float3 specular)
{
	//float3 lightDir = gLightPosW - worldPos;
	//float3 lightDir = -gLightDirW;
	float3 lightDir = -(dirLights[i].direction);

	float3 halfVector = normalize(normalize(lightDir) + normalize(viewDir));

    float3 h = normalize(halfVector);
    float3 l = normalize(lightDir);

	//ambient light computation
	ambient += (dirLights[i].ambient).rgb;
	
	//diffuse light computation
	//n = normalize(n);
	float nDotL = dot(n, l);
	diffuse += (dirLights[i].diffuse).rgb * nDotL;
	//diffuse += float3(1.0f, 0.0f, 0.0f);
	//diffuse = n;
	//specular light computation
	float nDotH = saturate(dot(n, h));
	float power = 0.0f;
	//only calculate specular if nDotL is not equal to 0
	if (nDotL != 0.0f)
	{
		if (hasSpecularMap)
		{
			//power = pow(nDotH, tex2D(gSpecularMap, texCoord).r);
			power = pow(nDotH, gSpecPower); 
		}
		else
		{
			power = pow(nDotH, gSpecPower); 
		}
	}
	specular += (dirLights[i].specular).rgb * power;

	//float power = (nDotL == 0.0f) ? 0.0f : pow(nDotH, gSpecPower);

	//might use this code to handle specular maps
	//     if (nDotVP == 0.0)
	//         pf = 0.0;
	//     else
	//         pf = pow(nDotHV, gl_FrontMaterial.shininess);
	//         pf = pow(nDotHV, texture2D(specMap, texCoord).r);

}

void DirectionalLightNormal(in int i, in float3 n, in float3 viewDir, in float3x3 tbnMatrix, in float2 texCoord,
								inout float3 ambient, inout float3 diffuse, inout float3 specular)
{
	//float3 lightDir = gLightPosW - worldPos;
	//float3 lightDir = -gLightDirW;
	float3 lightDir = -(dirLights[i].direction);
	
	float3 halfVector = (normalize(normalize(lightDir) + normalize(viewDir)))/2.0f;

	//float3 viewDirT = mul(halfVector, tbnMatrix);
	//float3 lightDirT = mul(lightDir, tbnMatrix);

    float3 h = normalize(halfVector);	// replaced viewDirT
    float3 l = normalize(lightDir);		// replaced lightDirT
	//n = mul(n, tbnMatrix);
	//ambient light computation
	ambient += (dirLights[i].ambient).rgb;
	
	//diffuse light computation
	float nDotL = saturate(dot(n, l));
	diffuse += (dirLights[i].diffuse).rgb * nDotL;
	//diffuse += float3(1.0f, 0.0f, 1.0f);
	//diffuse = n;
	//specular light computation
	float nDotH = saturate(dot(n, h));
	float power = 0.0f;
	//only calculate specular if nDotL is not equal to 0
	if (nDotL != 0.0f)
	{
		if (hasSpecularMap)
		{
			//power = pow(nDotH, tex2D(gSpecularMap, texCoord).r);
			power = pow(nDotH, 100); 
		}
		else
		{
			power = pow(nDotH, 100); 
		}
		
	}
	specular += (dirLights[i].specular).rgb * power;

//might use this code to handle specular maps
//     if (nDotVP == 0.0)
//         pf = 0.0;
//     else
//         pf = pow(nDotHV, gl_FrontMaterial.shininess);
//         pf = pow(nDotHV, texture2D(specMap, texCoord).r);

}



void PointLight(in int i, in float3 n, in float3 worldPos, in float3 viewDir, in float2 texCoord,
						inout float3 ambient, inout float3 diffuse, inout float3 specular)
{
	//float3 lightDirT = mul((dirLights[i].pos - worldPos) / dirLights[i].radius, tbnMatrix);
	//float3 atten = 1.0f - dot(lightDirT, lightDirT);

	float3 lightDir = normalize(dirLights[i].pos - worldPos);
	float distanceToLight = length(dirLights[i].pos - worldPos);
	float3 atten = saturate(1.0f - (distanceToLight/dirLights[i].radius));
	
	//n = -1*n;
	float3 viewDirT = (viewDir + lightDir) / 2.0f; // added new line
	//float3 viewDirT = mul(viewDir, tbnMatrix);
	//n = mul(n, tbnMatrix);
    float3 l =  normalize(worldPos - dirLights[i].pos);
    float3 h = normalize(viewDirT);

	//ambient light computation
	ambient += (dirLights[i].ambient).rgb * atten + globalAmbient;
	
	//make sure nDotL is clamped between 0 and 1
	float nDotL =  saturate(dot(-n,l));
	diffuse += (dirLights[i].diffuse).rgb * nDotL * atten;
	//diffuse += float3(1.0f, 0.0f, 0.0f);
	
	//specular light computation
	//make sure nDotH is clamped between 0 and 1
	float nDotH = saturate(dot(n, h));
	float power = 0.0f;
	
	//only calculate specular if nDotL is not equal to 0
	if (nDotL != 0.0f)
	{
		if (hasSpecularMap)
		{
			//power = pow(nDotH, gSpecPower); 
			power = pow(nDotH, 100); 
		}
		else
		{
			power = pow(nDotH, 100); 
		}
	}
	specular += (dirLights[i].specular).rgb * power * atten;
	//specular = 0;

	//might use this code to handle specular maps
	//     if (nDotVP == 0.0)
	//         pf = 0.0;
	//     else
	//         pf = pow(nDotHV, gl_FrontMaterial.shininess);
	//         pf = pow(nDotHV, texture2D(specMap, texCoord).r);

}

void PointLightNormal(in int i, in float3 n, in float3 worldPos, in float3 viewDir, in float3x3 tbnMatrix,
								in float2 texCoord, inout float3 ambient, inout float3 diffuse, inout float3 specular)
{// THIS IS THE FUNCTION I'M USING 

	//float3 lightDirT = mul((dirLights[i].pos - worldPos) / dirLights[i].radius, tbnMatrix);
	//float3 atten = 1.0f - dot(lightDirT, lightDirT);

	float3 lightDir = normalize(dirLights[i].pos - worldPos);
	float distanceToLight = length(dirLights[i].pos - worldPos);
	float3 atten = saturate(1.0f - (distanceToLight/dirLights[i].radius));
	
	//n = -1*n;
	float3 viewDirT = (viewDir + lightDir) / 2.0f; // added new line
	//float3 viewDirT = mul(viewDir, tbnMatrix);
	//n = mul(n, tbnMatrix);
    float3 l =  normalize(worldPos - dirLights[i].pos);
    float3 h = normalize(viewDirT);

	//ambient light computation
	ambient += (dirLights[i].ambient).rgb * atten + globalAmbient;
	
	//make sure nDotL is clamped between 0 and 1
	float nDotL =  saturate(dot(-n,l));
	diffuse += (dirLights[i].diffuse).rgb * nDotL * atten;
	//diffuse += float3(1.0f, 0.0f, 0.0f);
	
	//specular light computation
	//make sure nDotH is clamped between 0 and 1
	float nDotH = saturate(dot(n, h));
	float power = 0.0f;
	
	//only calculate specular if nDotL is not equal to 0
	if (nDotL != 0.0f)
	{
		if (hasSpecularMap)
		{
			//power = pow(nDotH, gSpecPower); 
			power = pow(nDotH, 100); 
		}
		else
		{
			power = pow(nDotH, 100); 
		}
	}
	specular += (dirLights[i].specular).rgb * power * atten;
}

VS_OUTPUT_TRUE VS_Blinn_Phong(VS_INPUT IN)
{
	//VS_OUTPUT_TRUE OUT;
	VS_OUTPUT_TRUE OUT = (VS_OUTPUT_TRUE)0;

	float3 Pos = IN.position;
    float3 Normal = IN.normal;
    //Normal = float3(0.0f, 1.0f, 0.0f);
    	
	TransformVertices(Pos, Normal);

	OUT.pos = mul( float4(Pos, 1.0f) , worldViewProj);
	OUT.normal = Normal;
	//OUT.normal = mul( float4(Normal, 0) , world );

	//transform vertex position to world space
	OUT.worldpos = Pos;
	//OUT.worldpos = mul(float4(Pos, 1.0f), world).xyz;
	//OUT.worldpos = mul(float4(IN.position, 1.0f), world).xyz;
	
	//OUT.viewDir = gEyePosW - OUT.worldpos;

	OUT.tex0 = IN.tex0;
        
	return OUT;
}

float4 PS_Blinn_Phong(VS_OUTPUT_TRUE IN) : COLOR
{
	float4 color;

	float3 ambient = float3(0, 0, 0);
	float3 diffuse = float3(0, 0, 0);
	float3 specular = float3(0, 0, 0);

	float3 n = normalize(IN.normal);
	
	float3 viewDir = gEyePosW - IN.worldpos;

	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional, point, or spotlight
	    if (dirLights[i].type == 3)
		{
    	    DirectionalLight(i, n, viewDir, IN.tex0, ambient, diffuse, specular);
		}
	    else if (dirLights[i].type == 1)
		{
			PointLight(i, n, IN.worldpos, viewDir, IN.tex0, ambient, diffuse, specular);
		}
	    //else
	        //SpotLight(i, eye, ecPosition3, normal, amb, diff, spec);
	}

	//ambient light computation
	ambient = saturate(ambient);
	diffuse = saturate(diffuse);
	specular = saturate(specular);
	ambient *= (gAmbientMtrl).rgb;
	diffuse *= (gDiffuseMtrl).rgb;
	specular *= (gSpecularMtrl).rgb;
		
	color.rgb = ambient + diffuse + specular;
	//color.rgb = float3(1.0f, 0.0f, 0.0f);
	color.a = gDiffuseMtrl.a;

	if (hasDiffuseMap)
	{
		return color * tex2D(TexS, IN.tex0);
	}
	else
	{
		//return float4(IN.normal, 1.0f);
		//return float4(0, 1, 1, 1);
		return color;
	}
	
	
	
}

float4 PS_Blinn_Phong_Alpha(VS_OUTPUT_TRUE IN) : COLOR
{
	float4 color;

	float3 ambient = float3(0, 0, 0);
	float3 diffuse = float3(0, 0, 0);
	float3 specular = float3(0, 0, 0);

	float3 n = normalize(IN.normal);

	float3 viewDir = gEyePosW - IN.worldpos;

	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional, point, or spotlight
	    if (dirLights[i].type == 3)
		{
    	    DirectionalLight(i, n, viewDir, IN.tex0, ambient, diffuse, specular);
		}
	    else if (dirLights[i].type == 1)
		{
			PointLight(i, n, IN.worldpos, viewDir, IN.tex0, ambient, diffuse, specular);
		}
	    //else
	        //SpotLight(i, eye, ecPosition3, normal, amb, diff, spec);
	}

	//ambient light computation
	ambient = saturate(ambient);
	diffuse = saturate(diffuse);
	specular = saturate(specular);
	ambient *= (gAmbientMtrl).rgb;
	diffuse *= (gDiffuseMtrl).rgb;
	specular *= (gSpecularMtrl).rgb;
		
	color.rgb = ambient + diffuse + specular;
	//color.rgb = float3(1.0f, 1.0f, 0.0f);
	if (hasDiffuseMap)
	{
		color *= tex2D(TexS, IN.tex0);
	}
	//float4 color2 = tex2D(TexS, IN.tex0);
	//float4 color2 = float4(IN.tex0.xy, 0, 1);
	color.a = tex2D(AlphaMapS, IN.tex0).r;
	//return float4(1, 1, 0, 1);
	return color;
}

float4 PS_Target_Reticule(VS_OUTPUT_TRUE IN) : COLOR
{
	float4 color = float4(1, 1, 1, 1);
	
	if (hasDiffuseMap)
	{
		color *= tex2D(TexS, IN.tex0);
	}
	//float4 color2 = tex2D(TexS, IN.tex0);
	//float4 color2 = float4(IN.tex0.xy, 0, 1);
	color.a = tex2D(AlphaMapS, IN.tex0).r;
	return color;
}

VS_OUTPUT_NORMAL_TRUE VS_NormalMap( VS_INPUT_NORMAL IN )
{
	VS_OUTPUT_NORMAL_TRUE OUT;

	float3 Pos = IN.position;
    float3 Normal = IN.normal;
    	
	TransformVertices(Pos, Normal);
	
	
	//OUT.worldpos = mul(float4(IN.position, 1.0f), world).xyz;
	//OUT.worldpos = mul(float4(Pos, 1.0f), world).xyz;
	OUT.worldpos = Pos;

	//OUT.viewDir = gEyePosW - OUT.worldpos;
		
	OUT.pos = mul(float4(Pos, 1.0f), worldViewProj);
	OUT.tex0 = IN.tex0;

	OUT.normal = IN.normal; // Change Here
	OUT.tangent = IN.tangent;
	OUT.binormal = IN.binormal;

	return OUT;
}

VS_OUTPUT_NORMAL_TRUE VS_NormalMapSkinned( VS_INPUT_SKIN_NORMAL IN , uniform int NumBones )
{
	VS_OUTPUT_NORMAL_TRUE OUT;

    float3 Pos = float3(0,0,0);
    float3 Normal = float3(0,0,0);
    TransformSkinnedVertices(Pos, Normal, IN.position, IN.normal, IN.BlendWeights, IN.BlendIndices, NumBones);

	//OUT.worldpos = mul(IN.position, world).xyz;
	//OUT.worldpos = mul(float4(Pos, 1.0f), world).xyz;
	OUT.worldpos = Pos;

	//OUT.viewDir = gEyePosW - OUT.worldpos;
		
	OUT.pos = mul(float4(Pos, 1.0f), worldViewProj);
	OUT.tex0 = IN.tex0;

	OUT.normal = IN.normal;
	OUT.tangent = IN.tangent;
	OUT.binormal = IN.binormal;

	return OUT;
}

float4 PS_NormalMap(VS_OUTPUT_NORMAL_TRUE IN) : COLOR
{	// USING THIS PIXEL SHADER 
	float4 color;
	
	float3 ambient = float3(0, 0, 0);
	float3 diffuse = float3(0, 0, 0);
	float3 specular = float3(0, 0, 0);

	float3 viewDir = gEyePosW - IN.worldpos;

	float3 n = normalize(tex2D(NormalMapS, IN.tex0).rgb * 2.0f - 1.0f);
	//float3 n = normalize(IN.normal);
	//n = mul(float4(n,1.0f),world).rgb;
	// Build TBN-basis.
	float3x3 TBN;
	TBN[0] = IN.tangent;
	TBN[1] = IN.binormal;
	TBN[2] = IN.normal;
	
	float3x3 tbnMatrix = transpose(TBN);

	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional, point, or spotlight
	    if (dirLights[i].type == 3)
		{
    	    DirectionalLightNormal(i, n, viewDir, tbnMatrix, IN.tex0, ambient, diffuse, specular);
	    }
		else if (dirLights[i].type == 1)
		{
			PointLightNormal(i, n, IN.worldpos, viewDir, tbnMatrix, IN.tex0, ambient, diffuse, specular);
		}
	    //else
	        //SpotLight(i, eye, ecPosition3, normal, amb, diff, spec);		
    }

	//ambient light computation
	//ambient = saturate(ambient);
	//diffuse = saturate(diffuse);
	//specular = saturate(specular);
	ambient *= (gAmbientMtrl).rgb;
	diffuse *= (gDiffuseMtrl).rgb;
	specular *= (gSpecularMtrl).rgb;
		
	color.rgb = ambient + diffuse + specular;
	//color.rgb = float3(1.0f, 0.0f, 1.0f); // CULPRIT !!!
	//color.rgb = (n + 1.0f) / 2.0f;
	//color.rgb = diffuse;
	color.a = gDiffuseMtrl.a;

	if (hasDiffuseMap)
	{
		return color * tex2D(TexS, IN.tex0);
		//return float4(0, 0, 1, 1);
	}
	else
	{
		//return float4(0, 1, 0, 1);
		return color;
	}
}

float4 PS_NormalMap_Alpha(VS_OUTPUT_NORMAL_TRUE IN) : COLOR
{
	float4 color;
	
	float3 ambient = float3(0, 0, 0);
	float3 diffuse = float3(0, 0, 0);
	float3 specular = float3(0, 0, 0);

	float3 n = normalize(tex2D(NormalMapS, IN.tex0).rgb * 2.0f - 1.0f);

	float3 viewDir = gEyePosW - IN.worldpos;

	// Build TBN-basis.
	float3x3 TBN;
	TBN[0] = IN.tangent;
	TBN[1] = IN.binormal;
	TBN[2] = IN.normal;
	
	float3x3 tbnMatrix = transpose(TBN);

	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional, point, or spotlight
	    if (dirLights[i].type == 3)
		{
    	    DirectionalLightNormal(i, n, viewDir, tbnMatrix, IN.tex0, ambient, diffuse, specular);
	    }
		else if (dirLights[i].type == 1)
		{
			PointLightNormal(i, n, IN.worldpos, viewDir, tbnMatrix, IN.tex0, ambient, diffuse, specular);
		}
	    //else
	        //SpotLight(i, eye, ecPosition3, normal, amb, diff, spec);		
    }

	//ambient light computation
	//ambient = saturate(ambient);
	//diffuse = saturate(diffuse);
	//specular = saturate(specular);
	ambient *= (gAmbientMtrl).rgb;
	diffuse *= (gDiffuseMtrl).rgb;
	specular *= (gSpecularMtrl).rgb;
		
	color.rgb = ambient + diffuse + specular;
	//color.rgb = float3(0.0f, 1.0f, 0.0f);
	//color.rgb = (n + 1.0f) / 2.0f;
	//color.rgb =  diffuse;

	if (hasDiffuseMap)
	{
		color *= tex2D(TexS, IN.tex0);
	}
	color.a = tex2D(AlphaMapS, IN.tex0).r;
	return color;
	//return float4(1, 0, 0, 1);

}

// Reduce color palette and darken based on distance from light source

float4 PS_Toon(VS_OUTPUT_NORMAL_TRUE IN) : COLOR
{

	float4 color = (float4)0;
	if (hasDiffuseMap)
	{
		color = tex2D(TexS, IN.tex0);
	}
	else
	{
		//return float4(IN.normal, 1.0f);
		//return float4(0, 1, 1, 1);
		color = float4(1, 1, 1, 1);
	}

	float3 n = normalize(IN.normal);

	float3 viewDir = gEyePosW - IN.worldpos;


	float3 lightDir;

	//iterate through lights of scene
    for (int i = 0; i < numLights; ++i)
    {
		//and calculate light calculate from this light based on whether
		//it is directional, point, or spotlight
	    if (dirLights[i].type == 3)
		{
    	    lightDir = normalize(-(dirLights[i].direction));
		}
	    //else
	        //SpotLight(i, eye, ecPosition3, normal, amb, diff, spec);
	}

	
	float intensity = max(0, dot(lightDir,n));

	// Reduce color palette by flooring values at discrete intervals.

	color.rgba -= (color.rgba %  0.1f); //discretePallete

	// Darken color based on intensity

	if (intensity > 0.95)
		color -= float4(0.0,0.0,0.0,0.0);
	else if (intensity > 0.5)
		color -= float4(0.2,0.2,0.2,0.0);
	else if (intensity > 0.25)
		color -= float4(0.4,0.4,0.4,0.0);
	else
		color -= float4(0.6,0.6,0.6,0.0);

	return color;
}

technique Static
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_Blinn_Phong();
		PixelShader = compile ps_3_0 PS_Blinn_Phong();
		
	}
}

technique AlphaStatic
{
	pass Pass0
	{
		
		VertexShader = compile vs_3_0 VS_Blinn_Phong();
		PixelShader = compile ps_3_0 PS_Blinn_Phong_Alpha();
		AlphaBlendEnable = True;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;		
	}
}

technique NormalStatic
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_NormalMap();
		//PixelShader = compile ps_3_0 PS_NormalMap();
		PixelShader = compile ps_3_0 PS_Toon();		
	}
}

technique NormalAlphaStatic
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_NormalMap();
		PixelShader = compile ps_3_0 PS_NormalMap_Alpha();
		AlphaBlendEnable = True;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;
	}
}

technique NormalSkinned
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_NormalMapSkinned( 4 );
		PixelShader = compile ps_3_0 PS_NormalMap();
		//PixelShader = compile ps_3_0 PS_Toon();
	}
}

technique NormalAlphaSkinned
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 VS_NormalMapSkinned( 4 );
		PixelShader = compile ps_3_0 PS_NormalMap_Alpha();
		AlphaBlendEnable = True;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;		
	}
}

technique TargetReticule
{
	pass Pass0
	{
		
		VertexShader = compile vs_3_0 VS_Blinn_Phong();
		PixelShader = compile ps_3_0 PS_Target_Reticule();
		AlphaBlendEnable = True;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;		
	}
}
